unit Mabcpdlg;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, StdCtrls, Buttons, Mask, DBCtrls, DBTables,
  MabObj;

type
   TdlgMabChgPasswd = class(TForm)
      btnOk: TBitBtn;
      btnCancel: TBitBtn;
      srcUser: TDataSource;
    grpMain: TGroupBox;
    lblOldPassword: TLabel;
    edtOldPassword: TEdit;
    lblPassword: TLabel;
    edtPassword: TEdit;
    lblVerify: TLabel;
    edtVerify: TEdit;
      procedure btnOkClick(Sender: TObject);
   private
      { Private declarations }
      PasswordField: string;

      procedure SetButtonStyle(AButtonStyle: TButtonStyle);
      procedure SetOldPasswordEdit(AOldPasswordEdit: TMabPasswordEdit);
      procedure SetPasswordEdit(APasswordEdit: TMabPasswordEdit);
      procedure SetVerifyEdit(AVerifyEdit: TMabPasswordEdit);
   public
      { Public declarations }
      constructor Create(AOwner: TComponent; ADataSet: TDataSet;
         APasswordField: string);

      function Execute: Boolean;

      property ButtonStyle: TButtonStyle write SetButtonStyle;
      property OldPasswordEdit: TMabPasswordEdit write SetOldPasswordEdit;
      property PasswordEdit: TMabPasswordEdit write SetPasswordEdit;
      property VerifyEdit: TMabPasswordEdit write SetVerifyEdit;
   end;

   TMabDBChgPasswdDlg = class(TComponent)
   private
      { Private declarations }
      FButtonStyle: TButtonStyle;
      FCaption: string;
      FFont: TFont;
      FOldPasswordEdit: TMabPasswordEdit;
      FParent: TForm;
      FParentFont: Boolean;
      FPasswordEdit: TMabPasswordEdit;
      FPasswordField: string;
      FUserTable: TTable;
      FVerifyEdit: TMabPasswordEdit;

      procedure SetFont(AFont: TFont);
      procedure FontChange(Sender: TObject);
      procedure SetParentFont(AParentFont: Boolean);
      procedure SetPasswordField(APasswordField: string);
   protected
      { Protected declarations }
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;

      function Execute: Boolean;

      property Parent: TForm read FParent;
   published
      { Published declarations }
      property ButtonStyle: TButtonStyle read FButtonStyle write FButtonStyle
         default bsAutoDetect;
      property Caption: string read FCaption write FCaption;
      property Font: TFont read FFont write SetFont;
      property OldPasswordEdit: TMabPasswordEdit read FOldPasswordEdit
         write FOldPasswordEdit;
      property ParentFont: Boolean read FParentFont write SetParentFont
         default True;
      property PasswordEdit: TMabPasswordEdit read FPasswordEdit
         write FPasswordEdit;
      property PasswordField: string read FPasswordField write SetPasswordField;
      property UserTable: TTable read FUserTable write FUserTable;
      property VerifyEdit: TMabPasswordEdit read FVerifyEdit write FVerifyEdit;
   end;

var
  dlgMabChgPasswd: TdlgMabChgPasswd;

implementation

{$R *.DFM}

constructor TdlgMabChgPasswd.Create(AOwner: TComponent; ADataSet: TDataSet;
   APasswordField: string);
begin
   inherited Create(AOwner);

   srcUser.DataSet := ADataSet;
   PasswordField := APasswordField;

   edtOldPassword.Height := 0;
   edtPassword.Height := 0;
   edtVerify.Height := 0;
end;

procedure TdlgMabChgPasswd.SetButtonStyle(AButtonStyle: TButtonStyle);
begin
   btnOk.Style := AButtonStyle;
   btnCancel.Style := AButtonStyle;
end;

procedure TdlgMabChgPasswd.SetOldPasswordEdit(AOldPasswordEdit: TMabPasswordEdit);
begin
   with AOldPasswordEdit do
   begin
      lblOldPassword.Caption := Prompt;
      edtOldPassword.CharCase := CharCase;
      edtOldPassword.MaxLength := MaxLength;
      edtOldPassword.PasswordChar := PasswordChar;
      edtOldPassword.Text := Text;
   end;
end;

procedure TdlgMabChgPasswd.SetPasswordEdit(APasswordEdit: TMabPasswordEdit);
begin
   with APasswordEdit do
   begin
      lblPassword.Caption := Prompt;
      edtPassword.CharCase := CharCase;
      edtPassword.MaxLength := MaxLength;
      edtPassword.PasswordChar := PasswordChar;
      edtPassword.Text := Text;
   end;
end;

procedure TdlgMabChgPasswd.SetVerifyEdit(AVerifyEdit: TMabPasswordEdit);
begin
   with AVerifyEdit do
   begin
      lblVerify.Caption := Prompt;
      edtVerify.CharCase := CharCase;
      edtVerify.MaxLength := MaxLength;
      edtVerify.PasswordChar := PasswordChar;
      edtVerify.Text := Text;
   end;
end;

procedure TdlgMabChgPasswd.btnOkClick(Sender: TObject);
begin
   if edtOldPassword.Text <> srcUser.DataSet.FieldByName(PasswordField).AsString then
   begin
      Application.MessageBox('Incorrect password!', 'Change Password', MB_ICONEXCLAMATION);
      ActiveControl := edtOldPassword;
      edtOldPassword.SelectAll;
      ModalResult := mrNone;
   end
   else if edtPassword.Text <> edtVerify.Text then
   begin
      Application.MessageBox('Password verification failed!', 'Change Password', MB_ICONEXCLAMATION);
      ActiveControl := edtVerify;
      edtVerify.SelectAll;
      ModalResult := mrNone;
   end
   else
      ModalResult := btnOk.ModalResult;
end;

function TdlgMabChgPasswd.Execute: Boolean;
begin
   Result := (ShowModal = mrOk);
end;

constructor TMabDBChgPasswdDlg.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   if Owner is TForm then
      FParent := TForm(Owner)
   else
      raise Exception.Create('Owner must be of type TForm');

   FButtonStyle := bsAutoDetect;
   FCaption := 'Change Password';

   FFont := TFont.Create;
   FFont.Assign(Parent.Font);
   FFont.OnChange := FontChange;
   FParentFont := True;

   FOldPasswordEdit := TMabPasswordEdit.Create;
   FOldPasswordEdit.Prompt := '&Old password:';

   FPasswordEdit := TMabPasswordEdit.Create;
   FPasswordEdit.Prompt := '&Password:';

   FVerifyEdit := TMabPasswordEdit.Create;
   FVerifyEdit.Prompt := '&Verify password:';

   FPasswordField := 'Password';
end;

destructor TMabDBChgPasswdDlg.Destroy;
begin
   FFont.Free;
   FOldPasswordEdit.Free;
   FPasswordEdit.Free;
   FVerifyEdit.Free;

   inherited Destroy;
end;

procedure TMabDBChgPasswdDlg.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = UserTable) then
      UserTable := nil;
end;

procedure TMabDBChgPasswdDlg.SetFont(AFont: TFont);
begin
   if AFont <> FFont then
      FFont.Assign(AFont);
end;

procedure TMabDBChgPasswdDlg.FontChange(Sender: TObject);
begin
   ParentFont := False;
end;

procedure TMabDBChgPasswdDlg.SetParentFont(AParentFont: Boolean);
begin
   if AParentFont <> ParentFont then
   begin
      if AParentFont then
         Font := Parent.Font;
      FParentFont := AParentFont;
   end;
end;

procedure TMabDBChgPasswdDlg.SetPasswordField(APasswordField: string);
begin
   if (UserTable <> nil) and UserTable.Active
   and (UserTable.FindField(APasswordField) = nil) then
      raise EDatabaseError.Create('Field does not exist in UserTable')
   else
      FPasswordField := APasswordField
end;

function TMabDBChgPasswdDlg.Execute: Boolean;
var
   ChgDlg: TdlgMabChgPasswd;
begin
   if UserTable = nil then
      raise EDatabaseError.Create('UserTable must be set')
   else
   begin
      Result := False;
      ChgDlg := TdlgMabChgPasswd.Create(Self, UserTable, PasswordField);
      with ChgDlg do
         try
            ButtonStyle     := Self.ButtonStyle;
            Caption         := Self.Caption;
            OldPasswordEdit := Self.OldPasswordEdit;
            PasswordEdit    := Self.PasswordEdit;
            VerifyEdit      := Self.VerifyEdit;
            Font.Name       := ' ';
            Font.Assign(Self.Font);

            UserTable.Edit;
            Result := Execute;
            if Result then
            begin
               UserTable.FieldByName(PasswordField).AsString := ChgDlg.edtPassword.Text;
               UserTable.Post
            end
            else
               UserTable.Cancel;
         finally
            Free;
         end;
   end;
end;

end.
